# Inventory ... this should probably be JSON file

SKLEARNED_CHAMPION_URLS = ['https://raw.githubusercontent.com/microprediction/sklearned/main/champion_models/tsa_aggressive_d0_ensemble_1_160.json']





